@@ -6,6 +6,7 @@ pub fn cli() -> App {
     subcommand("new")
         .about("Create a new cargo package at <path>")
         .arg(Arg::with_name("path").required(true))
+        .arg(opt("registry", "Registry to use").value_name("REGISTRY"))
         .arg_new_opts()
 }
 
@@ -14,13 +15,13 @@ pub fn exec(config: &mut Config, args: &ArgMatches) -> CliResult {
 
     ops::new(&opts, config)?;
     let path = args.value_of("path").unwrap();
-    let project_name = if let Some(name) = args.value_of("name") {
+    let package_name = if let Some(name) = args.value_of("name") {
         name
     } else {
         path
     };
     config
         .shell()
-        .status("Created", format!("{} `{}` project", opts.kind, project_name))?;
+        .status("Created", format!("{} `{}` package", opts.kind, package_name))?;
     Ok(())
 }
