@@ -6,8 +6,8 @@ use serde::{de, ser};
 use url::Url;
 
 use core::PackageId;
-use util::{ToSemver, ToUrl};
 use util::errors::{CargoResult, CargoResultExt};
+use util::{ToSemver, ToUrl};
 
 /// Some or all of the data required to identify a package:
 ///
@@ -77,9 +77,9 @@ impl PackageIdSpec {
     }
 
     /// Roughly equivalent to `PackageIdSpec::parse(spec)?.query(i)`
-    pub fn query_str<'a, I>(spec: &str, i: I) -> CargoResult<&'a PackageId>
+    pub fn query_str<I>(spec: &str, i: I) -> CargoResult<PackageId>
     where
-        I: IntoIterator<Item = &'a PackageId>,
+        I: IntoIterator<Item = PackageId>,
     {
         let spec = PackageIdSpec::parse(spec)
             .chain_err(|| format_err!("invalid package id specification: `{}`", spec))?;
@@ -88,7 +88,7 @@ impl PackageIdSpec {
 
     /// Convert a `PackageId` to a `PackageIdSpec`, which will have both the `Version` and `Url`
     /// fields filled in.
-    pub fn from_package_id(package_id: &PackageId) -> PackageIdSpec {
+    pub fn from_package_id(package_id: PackageId) -> PackageIdSpec {
         PackageIdSpec {
             name: package_id.name().to_string(),
             version: Some(package_id.version().clone()),
@@ -104,7 +104,8 @@ impl PackageIdSpec {
         let frag = url.fragment().map(|s| s.to_owned());
         url.set_fragment(None);
         let (name, version) = {
-            let mut path = url.path_segments()
+            let mut path = url
+                .path_segments()
                 .ok_or_else(|| format_err!("pkgid urls must have a path: {}", url))?;
             let path_name = path.next_back().ok_or_else(|| {
                 format_err!(
@@ -159,7 +160,7 @@ impl PackageIdSpec {
     }
 
     /// Checks whether the given `PackageId` matches the `PackageIdSpec`.
-    pub fn matches(&self, package_id: &PackageId) -> bool {
+    pub fn matches(&self, package_id: PackageId) -> bool {
         if self.name() != &*package_id.name() {
             return false;
         }
@@ -178,9 +179,9 @@ impl PackageIdSpec {
 
     /// Checks a list of `PackageId`s to find 1 that matches this `PackageIdSpec`. If 0, 2, or
     /// more are found, then this returns an error.
-    pub fn query<'a, I>(&self, i: I) -> CargoResult<&'a PackageId>
+    pub fn query<I>(&self, i: I) -> CargoResult<PackageId>
     where
-        I: IntoIterator<Item = &'a PackageId>,
+        I: IntoIterator<Item = PackageId>,
     {
         let mut ids = i.into_iter().filter(|p| self.matches(*p));
         let ret = match ids.next() {
@@ -211,7 +212,7 @@ impl PackageIdSpec {
             None => Ok(ret),
         };
 
-        fn minimize(msg: &mut String, ids: &[&PackageId], spec: &PackageIdSpec) {
+        fn minimize(msg: &mut String, ids: &[PackageId], spec: &PackageIdSpec) {
             let mut version_cnt = HashMap::new();
             for id in ids {
                 *version_cnt.entry(id.version()).or_insert(0) += 1;
@@ -275,10 +276,10 @@ impl<'de> de::Deserialize<'de> for PackageIdSpec {
 
 #[cfg(test)]
 mod tests {
-    use core::{PackageId, SourceId};
     use super::PackageIdSpec;
-    use url::Url;
+    use core::{PackageId, SourceId};
     use semver::Version;
+    use url::Url;
 
     #[test]
     fn good_parsing() {
@@ -367,12 +368,12 @@ mod tests {
     fn matching() {
         let url = Url::parse("http://example.com").unwrap();
         let sid = SourceId::for_registry(&url).unwrap();
-        let foo = PackageId::new("foo", "1.2.3", &sid).unwrap();
-        let bar = PackageId::new("bar", "1.2.3", &sid).unwrap();
+        let foo = PackageId::new("foo", "1.2.3", sid).unwrap();
+        let bar = PackageId::new("bar", "1.2.3", sid).unwrap();
 
-        assert!(PackageIdSpec::parse("foo").unwrap().matches(&foo));
-        assert!(!PackageIdSpec::parse("foo").unwrap().matches(&bar));
-        assert!(PackageIdSpec::parse("foo:1.2.3").unwrap().matches(&foo));
-        assert!(!PackageIdSpec::parse("foo:1.2.2").unwrap().matches(&foo));
+        assert!(PackageIdSpec::parse("foo").unwrap().matches(foo));
+        assert!(!PackageIdSpec::parse("foo").unwrap().matches(bar));
+        assert!(PackageIdSpec::parse("foo:1.2.3").unwrap().matches(foo));
+        assert!(!PackageIdSpec::parse("foo:1.2.2").unwrap().matches(foo));
     }
 }
