@@ -67,14 +67,14 @@ impl Profiles {
     /// workspace.
     pub fn get_profile(
         &self,
-        pkg_id: &PackageId,
+        pkg_id: PackageId,
         is_member: bool,
         unit_for: UnitFor,
         mode: CompileMode,
         release: bool,
     ) -> Profile {
         let maker = match mode {
-            CompileMode::Test => {
+            CompileMode::Test | CompileMode::Bench => {
                 if release {
                     &self.bench
                 } else {
@@ -95,7 +95,6 @@ impl Profiles {
                     &self.dev
                 }
             }
-            CompileMode::Bench => &self.bench,
             CompileMode::Doc { .. } => &self.doc,
         };
         let mut profile = maker.get_profile(Some(pkg_id), is_member, unit_for);
@@ -164,7 +163,7 @@ struct ProfileMaker {
 impl ProfileMaker {
     fn get_profile(
         &self,
-        pkg_id: Option<&PackageId>,
+        pkg_id: Option<PackageId>,
         is_member: bool,
         unit_for: UnitFor,
     ) -> Profile {
@@ -206,11 +205,13 @@ impl ProfileMaker {
                 .keys()
                 .filter_map(|key| match *key {
                     ProfilePackageSpec::All => None,
-                    ProfilePackageSpec::Spec(ref spec) => if spec.matches(pkg_id) {
-                        Some(spec)
-                    } else {
-                        None
-                    },
+                    ProfilePackageSpec::Spec(ref spec) => {
+                        if spec.matches(pkg_id) {
+                            Some(spec)
+                        } else {
+                            None
+                        }
+                    }
                 })
                 .collect();
             match matches.len() {
@@ -291,7 +292,7 @@ impl ProfileMaker {
 }
 
 fn merge_toml(
-    pkg_id: Option<&PackageId>,
+    pkg_id: Option<PackageId>,
     is_member: bool,
     unit_for: UnitFor,
     profile: &mut Profile,
@@ -314,11 +315,13 @@ fn merge_toml(
                 .iter()
                 .filter_map(|(key, spec_profile)| match *key {
                     ProfilePackageSpec::All => None,
-                    ProfilePackageSpec::Spec(ref s) => if s.matches(pkg_id) {
-                        Some(spec_profile)
-                    } else {
-                        None
-                    },
+                    ProfilePackageSpec::Spec(ref s) => {
+                        if s.matches(pkg_id) {
+                            Some(spec_profile)
+                        } else {
+                            None
+                        }
+                    }
                 });
             if let Some(spec_profile) = matches.next() {
                 merge_profile(profile, spec_profile);
@@ -371,7 +374,7 @@ fn merge_profile(profile: &mut Profile, toml: &TomlProfile) {
 
 /// Profile settings used to determine which compiler flags to use for a
 /// target.
-#[derive(Clone, Copy, Eq)]
+#[derive(Clone, Copy, Eq, PartialOrd, Ord)]
 pub struct Profile {
     pub name: &'static str,
     pub opt_level: InternedString,
@@ -523,7 +526,7 @@ impl Profile {
 }
 
 /// The link-time-optimization setting.
-#[derive(Clone, Copy, PartialEq, Eq, Debug, Hash)]
+#[derive(Clone, Copy, PartialEq, Eq, Debug, Hash, PartialOrd, Ord)]
 pub enum Lto {
     /// False = no LTO
     /// True = "Fat" LTO
@@ -587,27 +590,36 @@ impl UnitFor {
     pub fn with_for_host(self, for_host: bool) -> UnitFor {
         UnitFor {
             custom_build: self.custom_build,
-            panic_ok: self.panic_ok && !for_host
+            panic_ok: self.panic_ok && !for_host,
         }
     }
 
     /// Returns true if this unit is for a custom build script or one of its
     /// dependencies.
-    pub fn is_custom_build(&self) -> bool {
+    pub fn is_custom_build(self) -> bool {
         self.custom_build
     }
 
     /// Returns true if this unit is allowed to set the `panic` compiler flag.
-    pub fn is_panic_ok(&self) -> bool {
+    pub fn is_panic_ok(self) -> bool {
         self.panic_ok
     }
 
     /// All possible values, used by `clean`.
     pub fn all_values() -> &'static [UnitFor] {
         static ALL: [UnitFor; 3] = [
-            UnitFor { custom_build: false, panic_ok: true },
-            UnitFor { custom_build: true, panic_ok: false },
-            UnitFor { custom_build: false, panic_ok: false },
+            UnitFor {
+                custom_build: false,
+                panic_ok: true,
+            },
+            UnitFor {
+                custom_build: true,
+                panic_ok: false,
+            },
+            UnitFor {
+                custom_build: false,
+                panic_ok: false,
+            },
         ];
         &ALL
     }
