@@ -3,11 +3,11 @@ use std::collections::HashMap;
 use semver::VersionReq;
 use url::Url;
 
-use core::{Dependency, PackageId, Source, SourceId, SourceMap, Summary};
 use core::PackageSet;
-use util::{profile, Config};
-use util::errors::{CargoResult, CargoResultExt};
+use core::{Dependency, PackageId, Source, SourceId, SourceMap, Summary};
 use sources::config::SourceConfigMap;
+use util::errors::{CargoResult, CargoResultExt};
+use util::{profile, Config};
 
 /// Source of information about a group of packages.
 ///
@@ -22,8 +22,8 @@ pub trait Registry {
         Ok(ret)
     }
 
-    fn describe_source(&self, source: &SourceId) -> String;
-    fn is_replaced(&self, source: &SourceId) -> bool;
+    fn describe_source(&self, source: SourceId) -> String;
+    fn is_replaced(&self, source: SourceId) -> bool;
 }
 
 /// This structure represents a registry of known packages. It internally
@@ -102,8 +102,8 @@ impl<'cfg> PackageRegistry<'cfg> {
         PackageSet::new(package_ids, self.sources, self.config)
     }
 
-    fn ensure_loaded(&mut self, namespace: &SourceId, kind: Kind) -> CargoResult<()> {
-        match self.source_ids.get(namespace) {
+    fn ensure_loaded(&mut self, namespace: SourceId, kind: Kind) -> CargoResult<()> {
+        match self.source_ids.get(&namespace) {
             // We've previously loaded this source, and we've already locked it,
             // so we're not allowed to change it even if `namespace` has a
             // slightly different precise version listed.
@@ -138,8 +138,8 @@ impl<'cfg> PackageRegistry<'cfg> {
         Ok(())
     }
 
-    pub fn add_sources(&mut self, ids: &[SourceId]) -> CargoResult<()> {
-        for id in ids.iter() {
+    pub fn add_sources(&mut self, ids: impl IntoIterator<Item = SourceId>) -> CargoResult<()> {
+        for id in ids {
             self.ensure_loaded(id, Kind::Locked)?;
         }
         Ok(())
@@ -150,13 +150,13 @@ impl<'cfg> PackageRegistry<'cfg> {
     }
 
     fn add_source(&mut self, source: Box<Source + 'cfg>, kind: Kind) {
-        let id = source.source_id().clone();
+        let id = source.source_id();
         self.sources.insert(source);
-        self.source_ids.insert(id.clone(), (id, kind));
+        self.source_ids.insert(id, (id, kind));
     }
 
     pub fn add_override(&mut self, source: Box<Source + 'cfg>) {
-        self.overrides.push(source.source_id().clone());
+        self.overrides.push(source.source_id());
         self.add_source(source, Kind::Override);
     }
 
@@ -165,8 +165,9 @@ impl<'cfg> PackageRegistry<'cfg> {
         for dep in deps.iter() {
             trace!("\t-> {}", dep);
         }
-        let sub_map = self.locked
-            .entry(id.source_id().clone())
+        let sub_map = self
+            .locked
+            .entry(id.source_id())
             .or_insert_with(HashMap::new);
         let sub_vec = sub_map
             .entry(id.name().to_string())
@@ -200,9 +201,14 @@ impl<'cfg> PackageRegistry<'cfg> {
         // Remember that each dependency listed in `[patch]` has to resolve to
         // precisely one package, so that's why we're just creating a flat list
         // of summaries which should be the same length as `deps` above.
-        let unlocked_summaries = deps.iter()
+        let unlocked_summaries = deps
+            .iter()
             .map(|dep| {
-                debug!("registring a patch for `{}` with `{}`", url, dep.package_name());
+                debug!(
+                    "registring a patch for `{}` with `{}`",
+                    url,
+                    dep.package_name()
+                );
 
                 // Go straight to the source for resolving `dep`. Load it as we
                 // normally would and then ask it directly for the list of summaries
@@ -216,7 +222,8 @@ impl<'cfg> PackageRegistry<'cfg> {
                         )
                     })?;
 
-                let mut summaries = self.sources
+                let mut summaries = self
+                    .sources
                     .get_mut(dep.source_id())
                     .expect("loaded source not present")
                     .query_vec(dep)?
@@ -257,11 +264,7 @@ impl<'cfg> PackageRegistry<'cfg> {
         // we want to fill in the `patches_available` map (later used in the
         // `lock` method) and otherwise store the unlocked summaries in
         // `patches` to get locked in a future call to `lock_patches`.
-        let ids = unlocked_summaries
-            .iter()
-            .map(|s| s.package_id())
-            .cloned()
-            .collect();
+        let ids = unlocked_summaries.iter().map(|s| s.package_id()).collect();
         self.patches_available.insert(url.clone(), ids);
         self.patches.insert(url.clone(), unlocked_summaries);
 
@@ -289,14 +292,14 @@ impl<'cfg> PackageRegistry<'cfg> {
         &self.patches
     }
 
-    fn load(&mut self, source_id: &SourceId, kind: Kind) -> CargoResult<()> {
+    fn load(&mut self, source_id: SourceId, kind: Kind) -> CargoResult<()> {
         (|| {
             debug!("loading source {}", source_id);
             let source = self.source_config.load(source_id)?;
             assert_eq!(source.source_id(), source_id);
 
             if kind == Kind::Override {
-                self.overrides.push(source_id.clone());
+                self.overrides.push(source_id);
             }
             self.add_source(source, kind);
 
@@ -304,12 +307,12 @@ impl<'cfg> PackageRegistry<'cfg> {
             let _p = profile::start(format!("updating: {}", source_id));
             self.sources.get_mut(source_id).unwrap().update()
         })()
-            .chain_err(|| format_err!("Unable to update {}", source_id))?;
+        .chain_err(|| format_err!("Unable to update {}", source_id))?;
         Ok(())
     }
 
     fn query_overrides(&mut self, dep: &Dependency) -> CargoResult<Option<Summary>> {
-        for s in self.overrides.iter() {
+        for &s in self.overrides.iter() {
             let src = self.sources.get_mut(s).unwrap();
             let dep = Dependency::new_override(&*dep.package_name(), s);
             let mut results = src.query_vec(&dep)?;
@@ -532,14 +535,14 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
         Ok(())
     }
 
-    fn describe_source(&self, id: &SourceId) -> String {
+    fn describe_source(&self, id: SourceId) -> String {
         match self.sources.get(id) {
             Some(src) => src.describe(),
             None => id.to_string(),
         }
     }
 
-    fn is_replaced(&self, id: &SourceId) -> bool {
+    fn is_replaced(&self, id: SourceId) -> bool {
         match self.sources.get(id) {
             Some(src) => src.is_replaced(),
             None => false,
@@ -549,9 +552,9 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
 
 fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Summary) -> Summary {
     let pair = locked
-        .get(summary.source_id())
+        .get(&summary.source_id())
         .and_then(|map| map.get(&*summary.name()))
-        .and_then(|vec| vec.iter().find(|&&(ref id, _)| id == summary.package_id()));
+        .and_then(|vec| vec.iter().find(|&&(id, _)| id == summary.package_id()));
 
     trace!("locking summary of {}", summary.package_id());
 
@@ -561,7 +564,12 @@ fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Sum
         None => summary,
     };
     summary.map_dependencies(|dep| {
-        trace!("\t{}/{}/{}", dep.package_name(), dep.version_req(), dep.source_id());
+        trace!(
+            "\t{}/{}/{}",
+            dep.package_name(),
+            dep.version_req(),
+            dep.source_id()
+        );
 
         // If we've got a known set of overrides for this summary, then
         // one of a few cases can arise:
@@ -583,8 +591,8 @@ fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Sum
         // Cases 1/2 are handled by `matches_id` and case 3 is handled by
         // falling through to the logic below.
         if let Some(&(_, ref locked_deps)) = pair {
-            let locked = locked_deps.iter().find(|id| dep.matches_id(id));
-            if let Some(locked) = locked {
+            let locked = locked_deps.iter().find(|&&id| dep.matches_id(id));
+            if let Some(&locked) = locked {
                 trace!("\tfirst hit on {}", locked);
                 let mut dep = dep.clone();
                 dep.lock_to(locked);
@@ -596,10 +604,10 @@ fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Sum
         // all known locked packages to see if they match this dependency.
         // If anything does then we lock it to that and move on.
         let v = locked
-            .get(dep.source_id())
+            .get(&dep.source_id())
             .and_then(|map| map.get(&*dep.package_name()))
-            .and_then(|vec| vec.iter().find(|&&(ref id, _)| dep.matches_id(id)));
-        if let Some(&(ref id, _)) = v {
+            .and_then(|vec| vec.iter().find(|&&(id, _)| dep.matches_id(id)));
+        if let Some(&(id, _)) = v {
             trace!("\tsecond hit on {}", id);
             let mut dep = dep.clone();
             dep.lock_to(id);
@@ -610,16 +618,16 @@ fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Sum
         // this dependency.
         let v = patches.get(dep.source_id().url()).map(|vec| {
             let dep2 = dep.clone();
-            let mut iter = vec.iter().filter(move |p| {
-                dep2.matches_ignoring_source(p)
-            });
+            let mut iter = vec
+                .iter()
+                .filter(move |&&p| dep2.matches_ignoring_source(p));
             (iter.next(), iter)
         });
         if let Some((Some(patch_id), mut remaining)) = v {
             assert!(remaining.next().is_none());
             let patch_source = patch_id.source_id();
             let patch_locked = locked
-                .get(patch_source)
+                .get(&patch_source)
                 .and_then(|m| m.get(&*patch_id.name()))
                 .map(|list| list.iter().any(|&(ref id, _)| id == patch_id))
                 .unwrap_or(false);
