@@ -1,13 +1,14 @@
-use std::collections::{HashMap, HashSet};
+use std::collections::{BTreeMap, HashMap, HashSet};
 use std::rc::Rc;
 
 use core::interning::InternedString;
 use core::{Dependency, FeatureValue, PackageId, SourceId, Summary};
+use im_rc;
 use util::CargoResult;
 use util::Graph;
 
-use super::types::RegistryQueryer;
-use super::types::{ActivateResult, ConflictReason, DepInfo, GraphNode, Method, RcList};
+use super::errors::ActivateResult;
+use super::types::{ConflictReason, DepInfo, GraphNode, Method, RcList, RegistryQueryer};
 
 pub use super::encode::{EncodableDependency, EncodablePackageId, EncodableResolve};
 pub use super::encode::{Metadata, WorkspaceResolve};
@@ -19,12 +20,9 @@ pub use super::resolve::Resolve;
 // possible.
 #[derive(Clone)]
 pub struct Context {
-    // TODO: Both this and the two maps below are super expensive to clone. We should
-    //       switch to persistent hash maps if we can at some point or otherwise
-    //       make these much cheaper to clone in general.
     pub activations: Activations,
-    pub resolve_features: HashMap<PackageId, Rc<HashSet<InternedString>>>,
-    pub links: HashMap<InternedString, PackageId>,
+    pub resolve_features: im_rc::HashMap<PackageId, Rc<HashSet<InternedString>>>,
+    pub links: im_rc::HashMap<InternedString, PackageId>,
 
     // These are two cheaply-cloneable lists (O(1) clone) which are effectively
     // hash maps but are built up as "construction lists". We'll iterate these
@@ -36,16 +34,16 @@ pub struct Context {
     pub warnings: RcList<String>,
 }
 
-pub type Activations = HashMap<(InternedString, SourceId), Rc<Vec<Summary>>>;
+pub type Activations = im_rc::HashMap<(InternedString, SourceId), Rc<Vec<Summary>>>;
 
 impl Context {
     pub fn new() -> Context {
         Context {
             resolve_graph: RcList::new(),
-            resolve_features: HashMap::new(),
-            links: HashMap::new(),
+            resolve_features: im_rc::HashMap::new(),
+            links: im_rc::HashMap::new(),
             resolve_replacements: RcList::new(),
-            activations: HashMap::new(),
+            activations: im_rc::HashMap::new(),
             warnings: RcList::new(),
         }
     }
@@ -55,14 +53,15 @@ impl Context {
     /// Returns true if this summary with the given method is already activated.
     pub fn flag_activated(&mut self, summary: &Summary, method: &Method) -> CargoResult<bool> {
         let id = summary.package_id();
-        let prev = self.activations
-            .entry((id.name(), id.source_id().clone()))
+        let prev = self
+            .activations
+            .entry((id.name(), id.source_id()))
             .or_insert_with(|| Rc::new(Vec::new()));
         if !prev.iter().any(|c| c == summary) {
-            self.resolve_graph.push(GraphNode::Add(id.clone()));
+            self.resolve_graph.push(GraphNode::Add(id));
             if let Some(link) = summary.links() {
                 ensure!(
-                    self.links.insert(link, id.clone()).is_none(),
+                    self.links.insert(link, id).is_none(),
                     "Attempting to resolve a with more then one crate with the links={}. \n\
                      This will not build as is. Consider rebuilding the .lock file.",
                     &*link
@@ -85,13 +84,10 @@ impl Context {
         };
 
         let has_default_feature = summary.features().contains_key("default");
-        Ok(match self.resolve_features.get(id) {
+        Ok(match self.resolve_features.get(&id) {
             Some(prev) => {
-                features
-                    .iter()
-                    .all(|f| prev.contains(&InternedString::new(f)))
-                    && (!use_default || prev.contains(&InternedString::new("default"))
-                        || !has_default_feature)
+                features.iter().all(|f| prev.contains(f))
+                    && (!use_default || prev.contains("default") || !has_default_feature)
             }
             None => features.is_empty() && (!use_default || !has_default_feature),
         })
@@ -111,7 +107,8 @@ impl Context {
 
         // Next, transform all dependencies into a list of possible candidates
         // which can satisfy that dependency.
-        let mut deps = deps.into_iter()
+        let mut deps = deps
+            .into_iter()
             .map(|(dep, features)| {
                 let candidates = registry.query(&dep)?;
                 Ok((dep, candidates, Rc::new(features)))
@@ -129,14 +126,14 @@ impl Context {
 
     pub fn prev_active(&self, dep: &Dependency) -> &[Summary] {
         self.activations
-            .get(&(dep.name(), dep.source_id().clone()))
+            .get(&(dep.package_name(), dep.source_id()))
             .map(|v| &v[..])
             .unwrap_or(&[])
     }
 
-    fn is_active(&self, id: &PackageId) -> bool {
+    pub fn is_active(&self, id: PackageId) -> bool {
         self.activations
-            .get(&(id.name(), id.source_id().clone()))
+            .get(&(id.name(), id.source_id()))
             .map(|v| v.iter().any(|s| s.package_id() == id))
             .unwrap_or(false)
     }
@@ -145,13 +142,13 @@ impl Context {
     /// are still active
     pub fn is_conflicting(
         &self,
-        parent: Option<&PackageId>,
-        conflicting_activations: &HashMap<PackageId, ConflictReason>,
+        parent: Option<PackageId>,
+        conflicting_activations: &BTreeMap<PackageId, ConflictReason>,
     ) -> bool {
         conflicting_activations
             .keys()
-            .chain(parent)
-            .all(|id| self.is_active(id))
+            .chain(parent.as_ref())
+            .all(|&id| self.is_active(id))
     }
 
     /// Return all dependencies and the features we want from them.
@@ -179,18 +176,19 @@ impl Context {
         for dep in deps {
             // Skip optional dependencies, but not those enabled through a
             // feature
-            if dep.is_optional() && !reqs.deps.contains_key(&*dep.name()) {
+            if dep.is_optional() && !reqs.deps.contains_key(&dep.name_in_toml()) {
                 continue;
             }
             // So we want this dependency. Move the features we want from
             // `feature_deps` to `ret` and register ourselves as using this
             // name.
-            let base = reqs.deps.get(&*dep.name()).unwrap_or(&default_dep);
-            used_features.insert(dep.name().as_str());
+            let base = reqs.deps.get(&dep.name_in_toml()).unwrap_or(&default_dep);
+            used_features.insert(dep.name_in_toml());
             let always_required = !dep.is_optional()
-                && !s.dependencies()
+                && !s
+                    .dependencies()
                     .iter()
-                    .any(|d| d.is_optional() && d.name() == dep.name());
+                    .any(|d| d.is_optional() && d.name_in_toml() == dep.name_in_toml());
             if always_required && base.0 {
                 self.warnings.push(format!(
                     "Package `{}` does not have feature `{}`. It has a required dependency \
@@ -198,7 +196,7 @@ impl Context {
                      This is currently a warning to ease the transition, but it will become an \
                      error in the future.",
                     s.package_id(),
-                    dep.name()
+                    dep.name_in_toml()
                 ));
             }
             let mut base = base.1.clone();
@@ -217,7 +215,9 @@ impl Context {
         // package does not actually have those dependencies. We classified
         // them as dependencies in the first place because there is no such
         // feature, either.
-        let remaining = reqs.deps.keys()
+        let remaining = reqs
+            .deps
+            .keys()
             .cloned()
             .filter(|s| !used_features.contains(s))
             .collect::<Vec<_>>();
@@ -228,11 +228,9 @@ impl Context {
                     "Package `{}` does not have these features: `{}`",
                     s.package_id(),
                     features
-                ).into(),
-                Some(p) => (
-                    p.package_id().clone(),
-                    ConflictReason::MissingFeatures(features),
-                ).into(),
+                )
+                .into(),
+                Some(p) => (p.package_id(), ConflictReason::MissingFeatures(features)).into(),
             });
         }
 
@@ -242,12 +240,12 @@ impl Context {
 
             let set = Rc::make_mut(
                 self.resolve_features
-                    .entry(pkgid.clone())
+                    .entry(pkgid)
                     .or_insert_with(|| Rc::new(HashSet::new())),
             );
 
             for feature in reqs.used {
-                set.insert(InternedString::new(feature));
+                set.insert(feature);
             }
         }
 
@@ -258,7 +256,7 @@ impl Context {
         let mut replacements = HashMap::new();
         let mut cur = &self.resolve_replacements;
         while let Some(ref node) = cur.head {
-            let (k, v) = node.0.clone();
+            let (k, v) = node.0;
             replacements.insert(k, v);
             cur = &node.1;
         }
@@ -286,7 +284,7 @@ impl Context {
 /// dependency features in a Requirements object, returning it to the resolver.
 fn build_requirements<'a, 'b: 'a>(
     s: &'a Summary,
-    method: &'b Method
+    method: &'b Method,
 ) -> CargoResult<Requirements<'a>> {
     let mut reqs = Requirements::new(s);
 
@@ -296,10 +294,10 @@ fn build_requirements<'a, 'b: 'a>(
             all_features: true, ..
         } => {
             for key in s.features().keys() {
-                reqs.require_feature(key)?;
+                reqs.require_feature(*key)?;
             }
             for dep in s.dependencies().iter().filter(|d| d.is_optional()) {
-                reqs.require_dependency(dep.name().as_str());
+                reqs.require_dependency(dep.name_in_toml());
             }
         }
         Method::Required {
@@ -318,8 +316,8 @@ fn build_requirements<'a, 'b: 'a>(
             uses_default_features: true,
             ..
         } => {
-            if s.features().get("default").is_some() {
-                reqs.require_feature("default")?;
+            if s.features().contains_key("default") {
+                reqs.require_feature(InternedString::new("default"))?;
             }
         }
         Method::Required {
@@ -337,12 +335,12 @@ struct Requirements<'a> {
     // specified set of features enabled. The boolean indicates whether this
     // package was specifically requested (rather than just requesting features
     // *within* this package).
-    deps: HashMap<&'a str, (bool, Vec<InternedString>)>,
+    deps: HashMap<InternedString, (bool, Vec<InternedString>)>,
     // The used features set is the set of features which this local package had
     // enabled, which is later used when compiling to instruct the code what
     // features were enabled.
-    used: HashSet<&'a str>,
-    visited: HashSet<&'a str>,
+    used: HashSet<InternedString>,
+    visited: HashSet<InternedString>,
 }
 
 impl<'r> Requirements<'r> {
@@ -355,7 +353,7 @@ impl<'r> Requirements<'r> {
         }
     }
 
-    fn require_crate_feature(&mut self, package: &'r str, feat: InternedString) {
+    fn require_crate_feature(&mut self, package: InternedString, feat: InternedString) {
         self.used.insert(package);
         self.deps
             .entry(package)
@@ -364,7 +362,7 @@ impl<'r> Requirements<'r> {
             .push(feat);
     }
 
-    fn seen(&mut self, feat: &'r str) -> bool {
+    fn seen(&mut self, feat: InternedString) -> bool {
         if self.visited.insert(feat) {
             self.used.insert(feat);
             false
@@ -373,20 +371,21 @@ impl<'r> Requirements<'r> {
         }
     }
 
-    fn require_dependency(&mut self, pkg: &'r str) {
+    fn require_dependency(&mut self, pkg: InternedString) {
         if self.seen(pkg) {
             return;
         }
         self.deps.entry(pkg).or_insert((false, Vec::new())).0 = true;
     }
 
-    fn require_feature(&mut self, feat: &'r str) -> CargoResult<()> {
+    fn require_feature(&mut self, feat: InternedString) -> CargoResult<()> {
         if feat.is_empty() || self.seen(feat) {
             return Ok(());
         }
-        for fv in self.summary
+        for fv in self
+            .summary
             .features()
-            .get(feat)
+            .get(feat.as_str())
             .expect("must be a valid feature")
         {
             match *fv {
@@ -403,11 +402,12 @@ impl<'r> Requirements<'r> {
 
     fn require_value<'f>(&mut self, fv: &'f FeatureValue) -> CargoResult<()> {
         match fv {
-            FeatureValue::Feature(feat) => self.require_feature(feat.as_str()),
-            FeatureValue::Crate(dep) => Ok(self.require_dependency(dep.as_str())),
+            FeatureValue::Feature(feat) => self.require_feature(*feat)?,
+            FeatureValue::Crate(dep) => self.require_dependency(*dep),
             FeatureValue::CrateFeature(dep, dep_feat) => {
-                Ok(self.require_crate_feature(dep.as_str(), *dep_feat))
+                self.require_crate_feature(*dep, *dep_feat)
             }
-        }
+        };
+        Ok(())
     }
 }
