@@ -39,7 +39,9 @@ impl Summary {
         links: Option<impl AsRef<str>>,
         namespaced_features: bool,
     ) -> CargoResult<Summary>
-    where K: Borrow<str> + Ord + Display {
+    where
+        K: Borrow<str> + Ord + Display,
+    {
         for dep in dependencies.iter() {
             let feature = dep.name_in_toml();
             if !namespaced_features && features.get(&*feature).is_some() {
@@ -69,8 +71,8 @@ impl Summary {
         })
     }
 
-    pub fn package_id(&self) -> &PackageId {
-        &self.inner.package_id
+    pub fn package_id(&self) -> PackageId {
+        self.inner.package_id
     }
     pub fn name(&self) -> InternedString {
         self.package_id().name()
@@ -78,7 +80,7 @@ impl Summary {
     pub fn version(&self) -> &Version {
         self.package_id().version()
     }
-    pub fn source_id(&self) -> &SourceId {
+    pub fn source_id(&self) -> SourceId {
         self.package_id().source_id()
     }
     pub fn dependencies(&self) -> &[Dependency] {
@@ -119,7 +121,7 @@ impl Summary {
         self
     }
 
-    pub fn map_source(self, to_replace: &SourceId, replace_with: &SourceId) -> Summary {
+    pub fn map_source(self, to_replace: SourceId, replace_with: SourceId) -> Summary {
         let me = if self.package_id().source_id() == to_replace {
             let new_id = self.package_id().with_source_id(replace_with);
             self.override_id(new_id)
@@ -143,7 +145,9 @@ fn build_feature_map<K>(
     dependencies: &[Dependency],
     namespaced: bool,
 ) -> CargoResult<FeatureMap>
-where K: Borrow<str> + Ord + Display {
+where
+    K: Borrow<str> + Ord + Display,
+{
     use self::FeatureValue::*;
     let mut dep_map = HashMap::new();
     for dep in dependencies.iter() {
@@ -258,42 +262,46 @@ where K: Borrow<str> + Ord + Display {
                 // not recognized as a feature is pegged as a `Crate`. Here we handle the case
                 // where the dependency exists but is non-optional. It branches on namespaced
                 // just to provide the correct string for the crate dependency in the error.
-                (&Crate(ref dep), true, false) => if namespaced {
-                    bail!(
-                        "Feature `{}` includes `crate:{}` which is not an \
-                         optional dependency.\nConsider adding \
-                         `optional = true` to the dependency",
-                        feature,
-                        dep
-                    )
-                } else {
-                    bail!(
-                        "Feature `{}` depends on `{}` which is not an \
-                         optional dependency.\nConsider adding \
-                         `optional = true` to the dependency",
-                        feature,
-                        dep
-                    )
-                },
+                (&Crate(ref dep), true, false) => {
+                    if namespaced {
+                        bail!(
+                            "Feature `{}` includes `crate:{}` which is not an \
+                             optional dependency.\nConsider adding \
+                             `optional = true` to the dependency",
+                            feature,
+                            dep
+                        )
+                    } else {
+                        bail!(
+                            "Feature `{}` depends on `{}` which is not an \
+                             optional dependency.\nConsider adding \
+                             `optional = true` to the dependency",
+                            feature,
+                            dep
+                        )
+                    }
+                }
                 // If namespaced, the value was tagged as a dependency; if not namespaced,
                 // this could be anything not defined as a feature. This handles the case
                 // where no such dependency is actually defined; again, the branch on
                 // namespaced here is just to provide the correct string in the error.
-                (&Crate(ref dep), false, _) => if namespaced {
-                    bail!(
-                        "Feature `{}` includes `crate:{}` which is not a known \
-                         dependency",
-                        feature,
-                        dep
-                    )
-                } else {
-                    bail!(
-                        "Feature `{}` includes `{}` which is neither a dependency nor \
-                         another feature",
-                        feature,
-                        dep
-                    )
-                },
+                (&Crate(ref dep), false, _) => {
+                    if namespaced {
+                        bail!(
+                            "Feature `{}` includes `crate:{}` which is not a known \
+                             dependency",
+                            feature,
+                            dep
+                        )
+                    } else {
+                        bail!(
+                            "Feature `{}` includes `{}` which is neither a dependency nor \
+                             another feature",
+                            feature,
+                            dep
+                        )
+                    }
+                }
                 (&Crate(_), true, true) => {}
                 // If the value is a feature for one of the dependencies, bail out if no such
                 // dependency is actually defined in the manifest.
@@ -372,11 +380,13 @@ impl FeatureValue {
         use self::FeatureValue::*;
         match *self {
             Feature(ref f) => f.to_string(),
-            Crate(ref c) => if s.namespaced_features() {
-                format!("crate:{}", &c)
-            } else {
-                c.to_string()
-            },
+            Crate(ref c) => {
+                if s.namespaced_features() {
+                    format!("crate:{}", &c)
+                } else {
+                    c.to_string()
+                }
+            }
             CrateFeature(ref c, ref f) => [c.as_ref(), f.as_ref()].join("/"),
         }
     }
