@@ -5,15 +5,15 @@ use std::path::{Path, PathBuf};
 use filetime::FileTime;
 use git2;
 use glob::Pattern;
-use ignore::Match;
 use ignore::gitignore::GitignoreBuilder;
+use ignore::Match;
 
-use core::{Dependency, Package, PackageId, Source, SourceId, Summary};
 use core::source::MaybePackage;
+use core::{Dependency, Package, PackageId, Source, SourceId, Summary};
 use ops;
-use util::{self, internal, CargoResult};
 use util::paths;
 use util::Config;
+use util::{self, internal, CargoResult};
 
 pub struct PathSource<'cfg> {
     source_id: SourceId,
@@ -29,9 +29,9 @@ impl<'cfg> PathSource<'cfg> {
     ///
     /// This source will only return the package at precisely the `path`
     /// specified, and it will be an error if there's not a package at `path`.
-    pub fn new(path: &Path, id: &SourceId, config: &'cfg Config) -> PathSource<'cfg> {
+    pub fn new(path: &Path, source_id: SourceId, config: &'cfg Config) -> PathSource<'cfg> {
         PathSource {
-            source_id: id.clone(),
+            source_id,
             path: path.to_path_buf(),
             updated: false,
             packages: Vec::new(),
@@ -48,7 +48,7 @@ impl<'cfg> PathSource<'cfg> {
     ///
     /// Note that this should be used with care and likely shouldn't be chosen
     /// by default!
-    pub fn new_recursive(root: &Path, id: &SourceId, config: &'cfg Config) -> PathSource<'cfg> {
+    pub fn new_recursive(root: &Path, id: SourceId, config: &'cfg Config) -> PathSource<'cfg> {
         PathSource {
             recursive: true,
             ..PathSource::new(root, id, config)
@@ -78,10 +78,10 @@ impl<'cfg> PathSource<'cfg> {
         if self.updated {
             Ok(self.packages.clone())
         } else if self.recursive {
-            ops::read_packages(&self.path, &self.source_id, self.config)
+            ops::read_packages(&self.path, self.source_id, self.config)
         } else {
             let path = self.path.join("Cargo.toml");
-            let (pkg, _) = ops::read_package(&path, &self.source_id, self.config)?;
+            let (pkg, _) = ops::read_package(&path, self.source_id, self.config)?;
             Ok(vec![pkg])
         }
     }
@@ -127,13 +127,15 @@ impl<'cfg> PathSource<'cfg> {
                 .map_err(|e| format_err!("could not parse glob pattern `{}`: {}", p, e))
         };
 
-        let glob_exclude = pkg.manifest()
+        let glob_exclude = pkg
+            .manifest()
             .exclude()
             .iter()
             .map(|p| glob_parse(p))
             .collect::<Result<Vec<_>, _>>()?;
 
-        let glob_include = pkg.manifest()
+        let glob_include = pkg
+            .manifest()
             .include()
             .iter()
             .map(|p| glob_parse(p))
@@ -302,7 +304,8 @@ impl<'cfg> PathSource<'cfg> {
     ) -> CargoResult<Vec<PathBuf>> {
         warn!("list_files_git {}", pkg.package_id());
         let index = repo.index()?;
-        let root = repo.workdir()
+        let root = repo
+            .workdir()
             .ok_or_else(|| internal("Can't list files on a bare repository."))?;
         let pkg_path = pkg.root();
 
@@ -374,7 +377,8 @@ impl<'cfg> PathSource<'cfg> {
             if is_dir.unwrap_or_else(|| file_path.is_dir()) {
                 warn!("  found submodule {}", file_path.display());
                 let rel = util::without_prefix(&file_path, root).unwrap();
-                let rel = rel.to_str()
+                let rel = rel
+                    .to_str()
                     .ok_or_else(|| format_err!("invalid utf-8 filename: {}", rel.display()))?;
                 // Git submodules are currently only named through `/` path
                 // separators, explicitly not `\` which windows uses. Who knew?
@@ -398,8 +402,8 @@ impl<'cfg> PathSource<'cfg> {
 
         #[cfg(unix)]
         fn join(path: &Path, data: &[u8]) -> CargoResult<PathBuf> {
-            use std::os::unix::prelude::*;
             use std::ffi::OsStr;
+            use std::os::unix::prelude::*;
             Ok(path.join(<OsStr as OsStrExt>::from_bytes(data)))
         }
         #[cfg(windows)]
@@ -527,8 +531,8 @@ impl<'cfg> Source for PathSource<'cfg> {
         false
     }
 
-    fn source_id(&self) -> &SourceId {
-        &self.source_id
+    fn source_id(&self) -> SourceId {
+        self.source_id
     }
 
     fn update(&mut self) -> CargoResult<()> {
@@ -541,7 +545,7 @@ impl<'cfg> Source for PathSource<'cfg> {
         Ok(())
     }
 
-    fn download(&mut self, id: &PackageId) -> CargoResult<MaybePackage> {
+    fn download(&mut self, id: PackageId) -> CargoResult<MaybePackage> {
         trace!("getting packages; id={}", id);
 
         let pkg = self.packages.iter().find(|pkg| pkg.package_id() == id);
@@ -550,7 +554,7 @@ impl<'cfg> Source for PathSource<'cfg> {
             .ok_or_else(|| internal(format!("failed to find {} in path source", id)))
     }
 
-    fn finish_download(&mut self, _id: &PackageId, _data: Vec<u8>) -> CargoResult<Package> {
+    fn finish_download(&mut self, _id: PackageId, _data: Vec<u8>) -> CargoResult<Package> {
         panic!("no download should have started")
     }
 
