@@ -1,23 +1,24 @@
 use serde::ser;
-use serde_json::{self, Value};
+use serde_json::{self, value::RawValue};
 
-use core::{PackageId, Target, Profile};
+use core::{PackageId, Target};
 
 pub trait Message: ser::Serialize {
     fn reason(&self) -> &str;
 }
 
-pub fn emit<T: Message>(t: T) {
-    let mut json: Value = serde_json::to_value(&t).unwrap();
-    json["reason"] = json!(t.reason());
-    println!("{}", json);
+pub fn emit<T: Message>(t: &T) {
+    let json = serde_json::to_string(t).unwrap();
+    assert!(json.starts_with("{\""));
+    let reason = json!(t.reason());
+    println!("{{\"reason\":{},{}", reason, &json[1..]);
 }
 
 #[derive(Serialize)]
 pub struct FromCompiler<'a> {
-    pub package_id: &'a PackageId,
+    pub package_id: PackageId,
     pub target: &'a Target,
-    pub message: serde_json::Value,
+    pub message: Box<RawValue>,
 }
 
 impl<'a> Message for FromCompiler<'a> {
@@ -28,9 +29,9 @@ impl<'a> Message for FromCompiler<'a> {
 
 #[derive(Serialize)]
 pub struct Artifact<'a> {
-    pub package_id: &'a PackageId,
+    pub package_id: PackageId,
     pub target: &'a Target,
-    pub profile: &'a Profile,
+    pub profile: ArtifactProfile,
     pub features: Vec<String>,
     pub filenames: Vec<String>,
     pub fresh: bool,
@@ -42,12 +43,25 @@ impl<'a> Message for Artifact<'a> {
     }
 }
 
+/// This is different from the regular `Profile` to maintain backwards
+/// compatibility (in particular, `test` is no longer in `Profile`, but we
+/// still want it to be included here).
+#[derive(Serialize)]
+pub struct ArtifactProfile {
+    pub opt_level: &'static str,
+    pub debuginfo: Option<u32>,
+    pub debug_assertions: bool,
+    pub overflow_checks: bool,
+    pub test: bool,
+}
+
 #[derive(Serialize)]
 pub struct BuildScript<'a> {
-    pub package_id: &'a PackageId,
+    pub package_id: PackageId,
     pub linked_libs: &'a [String],
     pub linked_paths: &'a [String],
     pub cfgs: &'a [String],
+    pub env: &'a [(String, String)],
 }
 
 impl<'a> Message for BuildScript<'a> {
