@@ -265,12 +265,9 @@ fn cargo_subcommand_args() {
     cargo_process("foo bar -v --help")
         .env("PATH", &path)
         .with_stdout(
-            if cfg!(windows) { // weird edge-case w/ CWD & (windows vs unix)
-                format!(r#"[{:?}, "foo", "bar", "-v", "--help"]"#, cargo_foo_bin)
-            } else {
-                r#"["[CWD]/cargo-foo/target/debug/cargo-foo", "foo", "bar", "-v", "--help"]"#.to_string()
-            }
-        ).run();
+            r#"["[CWD]/cargo-foo/target/debug/cargo-foo[EXE]", "foo", "bar", "-v", "--help"]"#,
+        )
+        .run();
 }
 
 #[test]
