@@ -1,5 +1,4 @@
-use cargotest::support::{basic_bin_manifest, execs, main_file, project};
-use hamcrest::assert_that;
+use support::{basic_bin_manifest, main_file, project};
 
 static MANIFEST_OUTPUT: &'static str = r#"
 {
@@ -16,13 +15,15 @@ static MANIFEST_OUTPUT: &'static str = r#"
     "license": null,
     "license_file": null,
     "description": null,
+    "edition": "2015",
     "source":null,
     "dependencies":[],
     "targets":[{
         "kind":["bin"],
         "crate_types":["bin"],
+        "edition": "2015",
         "name":"foo",
-        "src_path":"[..][/]foo[/]src[/]foo.rs"
+        "src_path":"[..]/foo/src/foo.rs"
     }],
     "features":{},
     "manifest_path":"[..]Cargo.toml",
@@ -31,83 +32,72 @@ static MANIFEST_OUTPUT: &'static str = r#"
 
 #[test]
 fn cargo_read_manifest_path_to_cargo_toml_relative() {
-    let p = project("foo")
+    let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
         .build();
 
-    assert_that(
-        p.cargo("read-manifest")
-            .arg("--manifest-path")
-            .arg("foo/Cargo.toml")
-            .cwd(p.root().parent().unwrap()),
-        execs().with_status(0).with_json(MANIFEST_OUTPUT),
-    );
+    p.cargo("read-manifest --manifest-path foo/Cargo.toml")
+        .cwd(p.root().parent().unwrap())
+        .with_json(MANIFEST_OUTPUT)
+        .run();
 }
 
 #[test]
 fn cargo_read_manifest_path_to_cargo_toml_absolute() {
-    let p = project("foo")
+    let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
         .build();
 
-    assert_that(
-        p.cargo("read-manifest")
-            .arg("--manifest-path")
-            .arg(p.root().join("Cargo.toml"))
-            .cwd(p.root().parent().unwrap()),
-        execs().with_status(0).with_json(MANIFEST_OUTPUT),
-    );
+    p.cargo("read-manifest --manifest-path")
+        .arg(p.root().join("Cargo.toml"))
+        .cwd(p.root().parent().unwrap())
+        .with_json(MANIFEST_OUTPUT)
+        .run();
 }
 
 #[test]
 fn cargo_read_manifest_path_to_cargo_toml_parent_relative() {
-    let p = project("foo")
+    let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
         .build();
 
-    assert_that(
-        p.cargo("read-manifest")
-            .arg("--manifest-path")
-            .arg("foo")
-            .cwd(p.root().parent().unwrap()),
-        execs().with_status(101).with_stderr(
+    p.cargo("read-manifest --manifest-path foo")
+        .cwd(p.root().parent().unwrap())
+        .with_status(101)
+        .with_stderr(
             "[ERROR] the manifest-path must be \
              a path to a Cargo.toml file",
-        ),
-    );
+        ).run();
 }
 
 #[test]
 fn cargo_read_manifest_path_to_cargo_toml_parent_absolute() {
-    let p = project("foo")
+    let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
         .build();
 
-    assert_that(
-        p.cargo("read-manifest")
-            .arg("--manifest-path")
-            .arg(p.root())
-            .cwd(p.root().parent().unwrap()),
-        execs().with_status(101).with_stderr(
+    p.cargo("read-manifest --manifest-path")
+        .arg(p.root())
+        .cwd(p.root().parent().unwrap())
+        .with_status(101)
+        .with_stderr(
             "[ERROR] the manifest-path must be \
              a path to a Cargo.toml file",
-        ),
-    );
+        ).run();
 }
 
 #[test]
 fn cargo_read_manifest_cwd() {
-    let p = project("foo")
+    let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
         .build();
 
-    assert_that(
-        p.cargo("read-manifest").cwd(p.root()),
-        execs().with_status(0).with_json(MANIFEST_OUTPUT),
-    );
+    p.cargo("read-manifest")
+        .with_json(MANIFEST_OUTPUT)
+        .run();
 }
