@@ -3,68 +3,6 @@ use support::paths;
 use support::registry::Package;
 use support::{basic_manifest, project};
 
-#[test]
-fn gated() {
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-
-            [dependencies]
-            bar = { package = "foo", version = "0.1" }
-        "#,
-        ).file("src/lib.rs", "")
-        .build();
-
-    p.cargo("build")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr(
-            "\
-error: failed to parse manifest at `[..]`
-
-Caused by:
-  feature `rename-dependency` is required
-
-consider adding `cargo-features = [\"rename-dependency\"]` to the manifest
-",
-        ).run();
-
-    let p = project()
-        .at("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-
-            [dependencies]
-            bar = { version = "0.1", package = "baz" }
-        "#,
-        ).file("src/lib.rs", "")
-        .build();
-
-    p.cargo("build")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr(
-            "\
-error: failed to parse manifest at `[..]`
-
-Caused by:
-  feature `rename-dependency` is required
-
-consider adding `cargo-features = [\"rename-dependency\"]` to the manifest
-",
-        ).run();
-}
-
 #[test]
 fn rename_dependency() {
     Package::new("bar", "0.1.0").publish();
@@ -74,8 +12,6 @@ fn rename_dependency() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["rename-dependency"]
-
             [project]
             name = "foo"
             version = "0.0.1"
@@ -88,7 +24,7 @@ fn rename_dependency() {
         ).file("src/lib.rs", "extern crate bar; extern crate baz;")
         .build();
 
-    p.cargo("build").masquerade_as_nightly_cargo().run();
+    p.cargo("build").run();
 }
 
 #[test]
@@ -97,8 +33,6 @@ fn rename_with_different_names() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["rename-dependency"]
-
             [project]
             name = "foo"
             version = "0.0.1"
@@ -122,7 +56,7 @@ fn rename_with_different_names() {
         ).file("bar/src/lib.rs", "")
         .build();
 
-    p.cargo("build").masquerade_as_nightly_cargo().run();
+    p.cargo("build").run();
 }
 
 #[test]
@@ -148,8 +82,7 @@ fn lots_of_names() {
             "Cargo.toml",
             &format!(
                 r#"
-                cargo-features = ["alternative-registries", "rename-dependency"]
-
+                cargo-features = ["alternative-registries"]
                 [package]
                 name = "test"
                 version = "0.1.0"
@@ -196,8 +129,6 @@ fn rename_and_patch() {
         .file(
             "Cargo.toml",
             r#"
-                cargo-features = ["rename-dependency"]
-
                 [package]
                 name = "test"
                 version = "0.1.0"
@@ -216,7 +147,7 @@ fn rename_and_patch() {
         .file("foo/src/lib.rs", "pub fn foo() {}")
         .build();
 
-    p.cargo("build -v").masquerade_as_nightly_cargo().run();
+    p.cargo("build -v").run();
 }
 
 #[test]
@@ -227,8 +158,6 @@ fn rename_twice() {
         .file(
             "Cargo.toml",
             r#"
-                cargo-features = ["rename-dependency"]
-
                 [package]
                 name = "test"
                 version = "0.1.0"
@@ -243,12 +172,12 @@ fn rename_twice() {
         .build();
 
     p.cargo("build -v")
-        .masquerade_as_nightly_cargo()
         .with_status(101)
         .with_stderr(
             "\
 [UPDATING] `[..]` index
-[DOWNLOADING] foo v0.1.0 (registry [..])
+[DOWNLOADING] crates ...
+[DOWNLOADED] foo v0.1.0 (registry [..])
 error: multiple dependencies listed for the same crate must all have the same \
 name, but the dependency on `foo v0.1.0` is listed as having different names
 ",
@@ -263,8 +192,6 @@ fn rename_affects_fingerprint() {
         .file(
             "Cargo.toml",
             r#"
-                cargo-features = ["rename-dependency"]
-
                 [package]
                 name = "test"
                 version = "0.1.0"
@@ -276,13 +203,11 @@ fn rename_affects_fingerprint() {
         ).file("src/lib.rs", "extern crate foo;")
         .build();
 
-    p.cargo("build -v").masquerade_as_nightly_cargo().run();
+    p.cargo("build -v").run();
 
     p.change_file(
         "Cargo.toml",
         r#"
-                cargo-features = ["rename-dependency"]
-
                 [package]
                 name = "test"
                 version = "0.1.0"
@@ -294,7 +219,6 @@ fn rename_affects_fingerprint() {
     );
 
     p.cargo("build -v")
-        .masquerade_as_nightly_cargo()
         .with_status(101)
         .run();
 }
@@ -308,8 +232,6 @@ fn can_run_doc_tests() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["rename-dependency"]
-
             [project]
             name = "foo"
             version = "0.0.1"
@@ -327,14 +249,13 @@ fn can_run_doc_tests() {
         ).build();
 
     foo.cargo("test -v")
-        .masquerade_as_nightly_cargo()
         .with_stderr_contains(
             "\
 [DOCTEST] foo
 [RUNNING] `rustdoc --test [CWD]/src/lib.rs \
         [..] \
-        --extern baz=[CWD]/target/debug/deps/libbar-[..].rlib \
         --extern bar=[CWD]/target/debug/deps/libbar-[..].rlib \
+        --extern baz=[CWD]/target/debug/deps/libbar-[..].rlib \
         [..]`
 ",
         ).run();
@@ -362,8 +283,6 @@ fn features_still_work() {
         .file(
             "a/Cargo.toml",
             r#"
-                cargo-features = ["rename-dependency"]
-
                 [package]
                 name = "p1"
                 version = "0.1.0"
@@ -376,8 +295,6 @@ fn features_still_work() {
         .file(
             "b/Cargo.toml",
             r#"
-                cargo-features = ["rename-dependency"]
-
                 [package]
                 name = "p2"
                 version = "0.1.0"
@@ -392,7 +309,7 @@ fn features_still_work() {
         ).file("b/src/lib.rs", "extern crate b;")
         .build();
 
-    p.cargo("build -v").masquerade_as_nightly_cargo().run();
+    p.cargo("build -v").run();
 }
 
 #[test]
@@ -404,7 +321,6 @@ fn features_not_working() {
         .file(
             "Cargo.toml",
             r#"
-                cargo-features = ["rename-dependency"]
                 [package]
                 name = "test"
                 version = "0.1.0"
@@ -421,7 +337,6 @@ fn features_not_working() {
         .build();
 
     p.cargo("build -v")
-        .masquerade_as_nightly_cargo()
         .with_status(101)
         .with_stderr(
             "\
@@ -432,3 +347,26 @@ Caused by:
 ",
         ).run();
 }
+
+#[test]
+fn rename_with_dash() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "qwerty"
+                version = "0.1.0"
+
+                [dependencies]
+                foo-bar = { path = 'a', package = 'a' }
+            "#,
+        )
+        .file("src/lib.rs", "extern crate foo_bar;")
+        .file("a/Cargo.toml", &basic_manifest("a", "0.1.0"))
+        .file("a/src/lib.rs", "")
+        .build();
+
+    p.cargo("build")
+        .run();
+}
