@@ -1,6 +1,6 @@
 use std::env;
 
-use cargo::core::dependency::Kind::Development;
+use cargo::core::dependency::Kind;
 use cargo::core::{enable_nightly_features, Dependency};
 use cargo::util::Config;
 
@@ -15,6 +15,13 @@ use support::resolver::{
 use proptest::collection::vec;
 use proptest::prelude::*;
 
+/// NOTE: proptest is a form of fuzz testing. It generates random input and makes shore that
+/// certain universal truths are upheld. Therefore, it can pass when there is a problem,
+/// but if it fails then there really is something wrong. When testing something as
+/// complicated as the resolver, the problems can be very subtle and hard to generate.
+/// We have had a history of these tests only failing on PRs long after a bug is introduced.
+/// If you have one of these test fail please report it on #6258,
+/// and if you did not change the resolver then feel free to retry without concern.
 proptest! {
     #![proptest_config(ProptestConfig {
         // Note that this is a little low in terms of cases we'd like to test,
@@ -33,6 +40,8 @@ proptest! {
             },
         .. ProptestConfig::default()
     })]
+
+    /// NOTE: if you think this test has failed spuriously see the note at the top of this macro.
     #[test]
     fn passes_validation(
         PrettyPrintRegistry(input) in registry_strategy(50, 20, 60)
@@ -49,6 +58,58 @@ proptest! {
             );
         }
     }
+
+    /// NOTE: if you think this test has failed spuriously see the note at the top of this macro.
+    #[test]
+    fn minimum_version_errors_the_same(
+            PrettyPrintRegistry(input) in registry_strategy(50, 20, 60)
+    ) {
+        enable_nightly_features();
+
+        let mut config = Config::default().unwrap();
+        config
+            .configure(
+                1,
+                None,
+                &None,
+                false,
+                false,
+                &None,
+                &["minimal-versions".to_string()],
+            )
+            .unwrap();
+
+        let reg = registry(input.clone());
+        // there is only a small chance that eny one
+        // crate will be interesting.
+        // So we try some of the most complicated.
+        for this in input.iter().rev().take(10) {
+            // minimal-versions change what order the candidates
+            // are tried but not the existence of a solution
+            let res = resolve(
+                &pkg_id("root"),
+                vec![dep_req(&this.name(), &format!("={}", this.version()))],
+                &reg,
+            );
+
+            let mres = resolve_with_config(
+                &pkg_id("root"),
+                vec![dep_req(&this.name(), &format!("={}", this.version()))],
+                &reg,
+                Some(&config),
+            );
+
+            prop_assert_eq!(
+                res.is_ok(),
+                mres.is_ok(),
+                "minimal-versions and regular resolver disagree about weather `{} = \"={}\"` can resolve",
+                this.name(),
+                this.version()
+            )
+        }
+    }
+
+    /// NOTE: if you think this test has failed spuriously see the note at the top of this macro.
     #[test]
     fn limited_independence_of_irrelevant_alternatives(
         PrettyPrintRegistry(input) in registry_strategy(50, 20, 60),
@@ -72,7 +133,7 @@ proptest! {
                     let not_selected: Vec<_> = input
                         .iter()
                         .cloned()
-                        .filter(|x| !r.contains(x.package_id()))
+                        .filter(|x| !r.contains(&x.package_id()))
                         .collect();
                     if !not_selected.is_empty() {
                         let indexs_to_unpublish: Vec<_> = indexs_to_unpublish.iter().map(|x| x.get(&not_selected)).collect();
@@ -219,15 +280,15 @@ fn test_resolving_with_same_name() {
 #[test]
 fn test_resolving_with_dev_deps() {
     let reg = registry(vec![
-        pkg!("foo" => ["bar", dep_kind("baz", Development)]),
-        pkg!("baz" => ["bat", dep_kind("bam", Development)]),
+        pkg!("foo" => ["bar", dep_kind("baz", Kind::Development)]),
+        pkg!("baz" => ["bat", dep_kind("bam", Kind::Development)]),
         pkg!("bar"),
         pkg!("bat"),
     ]);
 
     let res = resolve(
         &pkg_id("root"),
-        vec![dep("foo"), dep_kind("baz", Development)],
+        vec![dep("foo"), dep_kind("baz", Kind::Development)],
         &reg,
     )
     .unwrap();
@@ -372,14 +433,12 @@ fn resolving_incompat_versions() {
         pkg!("bar" => [dep_req("foo", "=1.0.2")]),
     ]);
 
-    assert!(
-        resolve(
-            &pkg_id("root"),
-            vec![dep_req("foo", "=1.0.1"), dep("bar")],
-            &reg
-        )
-        .is_err()
-    );
+    assert!(resolve(
+        &pkg_id("root"),
+        vec![dep_req("foo", "=1.0.1"), dep("bar")],
+        &reg
+    )
+    .is_err());
 }
 
 #[test]
@@ -598,13 +657,13 @@ fn resolving_with_many_equivalent_backtracking() {
         let next = format!("level{}", l + 1);
         for i in 1..BRANCHING_FACTOR {
             let vsn = format!("1.0.{}", i);
-            reglist.push(pkg!((name.as_str(), vsn.as_str()) => [dep_req(next.as_str(), "*")]));
+            reglist.push(pkg!((name.as_str(), vsn.as_str()) => [dep(next.as_str())]));
         }
     }
 
     let reg = registry(reglist.clone());
 
-    let res = resolve(&pkg_id("root"), vec![dep_req("level0", "*")], &reg);
+    let res = resolve(&pkg_id("root"), vec![dep("level0")], &reg);
 
     assert!(res.is_err());
 
@@ -614,7 +673,7 @@ fn resolving_with_many_equivalent_backtracking() {
 
     let reg = registry(reglist.clone());
 
-    let res = resolve(&pkg_id("root"), vec![dep_req("level0", "*")], &reg).unwrap();
+    let res = resolve(&pkg_id("root"), vec![dep("level0")], &reg).unwrap();
 
     assert_contains(&res, &names(&[("root", "1.0.0"), ("level0", "1.0.0")]));
 
@@ -629,7 +688,7 @@ fn resolving_with_many_equivalent_backtracking() {
 
     let res = resolve(
         &pkg_id("root"),
-        vec![dep_req("level0", "*"), dep_req("constrained", "*")],
+        vec![dep("level0"), dep("constrained")],
         &reg,
     )
     .unwrap();
@@ -647,7 +706,7 @@ fn resolving_with_many_equivalent_backtracking() {
 
     let res = resolve(
         &pkg_id("root"),
-        vec![dep_req("level0", "1.0.1"), dep_req("constrained", "*")],
+        vec![dep_req("level0", "1.0.1"), dep("constrained")],
         &reg,
     )
     .unwrap();
@@ -686,7 +745,7 @@ fn resolving_with_deep_traps() {
         let next = format!("backtrack_trap{}", l + 1);
         for i in 1..BRANCHING_FACTOR {
             let vsn = format!("1.0.{}", i);
-            reglist.push(pkg!((name.as_str(), vsn.as_str()) => [dep_req(next.as_str(), "*")]));
+            reglist.push(pkg!((name.as_str(), vsn.as_str()) => [dep(next.as_str())]));
         }
     }
     {
@@ -708,7 +767,7 @@ fn resolving_with_deep_traps() {
 
     let res = resolve(
         &pkg_id("root"),
-        vec![dep_req("backtrack_trap0", "*"), dep_req("cloaking", "*")],
+        vec![dep("backtrack_trap0"), dep("cloaking")],
         &reg,
     );
 
@@ -729,7 +788,7 @@ fn resolving_with_constrained_cousins_backtrack() {
         let next = format!("backtrack_trap{}", l + 1);
         for i in 1..BRANCHING_FACTOR {
             let vsn = format!("1.0.{}", i);
-            reglist.push(pkg!((name.as_str(), vsn.as_str()) => [dep_req(next.as_str(), "*")]));
+            reglist.push(pkg!((name.as_str(), vsn.as_str()) => [dep(next.as_str())]));
         }
     }
     {
@@ -761,9 +820,9 @@ fn resolving_with_constrained_cousins_backtrack() {
     let res = resolve(
         &pkg_id("root"),
         vec![
-            dep_req("backtrack_trap0", "*"),
+            dep("backtrack_trap0"),
             dep_req("constrained", "2.0.1"),
-            dep_req("cloaking", "*"),
+            dep("cloaking"),
         ],
         &reg,
     );
@@ -777,12 +836,12 @@ fn resolving_with_constrained_cousins_backtrack() {
         let next = format!("level{}", l + 1);
         for i in 1..BRANCHING_FACTOR {
             let vsn = format!("1.0.{}", i);
-            reglist.push(pkg!((name.as_str(), vsn.as_str()) => [dep_req(next.as_str(), "*")]));
+            reglist.push(pkg!((name.as_str(), vsn.as_str()) => [dep(next.as_str())]));
         }
     }
     reglist.push(
-        pkg!((format!("level{}", DEPTH).as_str(), "1.0.0") => [dep_req("backtrack_trap0", "*"),
-            dep_req("cloaking", "*")
+        pkg!((format!("level{}", DEPTH).as_str(), "1.0.0") => [dep("backtrack_trap0"),
+            dep("cloaking")
         ]),
     );
 
@@ -790,7 +849,7 @@ fn resolving_with_constrained_cousins_backtrack() {
 
     let res = resolve(
         &pkg_id("root"),
-        vec![dep_req("level0", "*"), dep_req("constrained", "2.0.1")],
+        vec![dep("level0"), dep_req("constrained", "2.0.1")],
         &reg,
     );
 
@@ -798,7 +857,7 @@ fn resolving_with_constrained_cousins_backtrack() {
 
     let res = resolve(
         &pkg_id("root"),
-        vec![dep_req("level0", "*"), dep_req("constrained", "2.0.0")],
+        vec![dep("level0"), dep_req("constrained", "2.0.0")],
         &reg,
     )
     .unwrap();
@@ -942,7 +1001,7 @@ fn incomplete_information_skiping() {
         input
             .iter()
             .cloned()
-            .filter(|x| &package_to_yank != x.package_id())
+            .filter(|x| package_to_yank != x.package_id())
             .collect(),
     );
     assert_eq!(input.len(), new_reg.len() + 1);
@@ -985,7 +1044,7 @@ fn incomplete_information_skiping_2() {
             dep("bad"),
         ]),
         pkg!(("h", "3.8.3") => [
-            dep_req("g", "*"),
+            dep("g"),
         ]),
         pkg!(("h", "6.8.3") => [
             dep("f"),
@@ -994,10 +1053,10 @@ fn incomplete_information_skiping_2() {
             dep_req("to_yank", "=8.8.1"),
         ]),
         pkg!("i" => [
-            dep_req("b", "*"),
-            dep_req("c", "*"),
-            dep_req("e", "*"),
-            dep_req("h", "*"),
+            dep("b"),
+            dep("c"),
+            dep("e"),
+            dep("h"),
         ]),
     ];
     let reg = registry(input.clone());
@@ -1011,7 +1070,7 @@ fn incomplete_information_skiping_2() {
         input
             .iter()
             .cloned()
-            .filter(|x| &package_to_yank != x.package_id())
+            .filter(|x| package_to_yank != x.package_id())
             .collect(),
     );
     assert_eq!(input.len(), new_reg.len() + 1);
@@ -1043,7 +1102,7 @@ fn incomplete_information_skiping_3() {
         ] },
         pkg!{("b", "2.0.2") => [
             dep_req("to_yank", "3.3.0"),
-            dep_req("a", "*"),
+            dep("a"),
         ] },
         pkg!{("b", "2.3.3") => [
             dep_req("to_yank", "3.3.0"),
@@ -1052,7 +1111,7 @@ fn incomplete_information_skiping_3() {
     ];
     let reg = registry(input.clone());
 
-    let res = resolve(&pkg_id("root"), vec![dep_req("b", "*")], &reg).unwrap();
+    let res = resolve(&pkg_id("root"), vec![dep("b")], &reg).unwrap();
     let package_to_yank = ("to_yank", "3.0.3").to_pkgid();
     // this package is not used in the resolution.
     assert!(!res.contains(&package_to_yank));
@@ -1061,12 +1120,12 @@ fn incomplete_information_skiping_3() {
         input
             .iter()
             .cloned()
-            .filter(|x| &package_to_yank != x.package_id())
+            .filter(|x| package_to_yank != x.package_id())
             .collect(),
     );
     assert_eq!(input.len(), new_reg.len() + 1);
     // it should still build
-    assert!(resolve(&pkg_id("root"), vec![dep_req("b", "*")], &new_reg).is_ok());
+    assert!(resolve(&pkg_id("root"), vec![dep("b")], &new_reg).is_ok());
 }
 
 #[test]
@@ -1113,3 +1172,72 @@ fn hard_equality() {
         &names(&[("root", "1.0.0"), ("foo", "1.0.0"), ("bar", "1.0.0")]),
     );
 }
+
+#[test]
+fn large_conflict_cache() {
+    let mut input = vec![
+        pkg!(("last", "0.0.0") => [dep("bad")]), // just to make sure last is less constrained
+    ];
+    let mut root_deps = vec![dep("last")];
+    const NUM_VERSIONS: u8 = 3;
+    for name in 0..=NUM_VERSIONS {
+        // a large number of conflicts can easily be generated by a sys crate.
+        let sys_name = format!("{}-sys", (b'a' + name) as char);
+        let in_len = input.len();
+        input.push(pkg!(("last", format!("{}.0.0", in_len)) => [dep_req(&sys_name, "=0.0.0")]));
+        root_deps.push(dep_req(&sys_name, ">= 0.0.1"));
+
+        // a large number of conflicts can also easily be generated by a major release version.
+        let plane_name = format!("{}", (b'a' + name) as char);
+        let in_len = input.len();
+        input.push(pkg!(("last", format!("{}.0.0", in_len)) => [dep_req(&plane_name, "=1.0.0")]));
+        root_deps.push(dep_req(&plane_name, ">= 1.0.1"));
+
+        for i in 0..=NUM_VERSIONS {
+            input.push(pkg!((&sys_name, format!("{}.0.0", i))));
+            input.push(pkg!((&plane_name, format!("1.0.{}", i))));
+        }
+    }
+    let reg = registry(input);
+    let _ = resolve(&pkg_id("root"), root_deps, &reg);
+}
+
+#[test]
+fn conflict_store_bug() {
+    let input = vec![
+        pkg!(("A", "0.0.3")),
+        pkg!(("A", "0.0.5")),
+        pkg!(("A", "0.0.9") => [dep("bad"),]),
+        pkg!(("A", "0.0.10") => [dep("bad"),]),
+        pkg!(("L-sys", "0.0.1") => [dep("bad"),]),
+        pkg!(("L-sys", "0.0.5")),
+        pkg!(("R", "0.0.4") => [
+            dep_req("L-sys", "= 0.0.5"),
+        ]),
+        pkg!(("R", "0.0.6")),
+        pkg!(("a-sys", "0.0.5")),
+        pkg!(("a-sys", "0.0.11")),
+        pkg!(("c", "0.0.12") => [
+            dep_req("R", ">= 0.0.3, <= 0.0.4"),
+        ]),
+        pkg!(("c", "0.0.13") => [
+            dep_req("a-sys", ">= 0.0.8, <= 0.0.11"),
+        ]),
+        pkg!(("c0", "0.0.6") => [
+            dep_req("L-sys", "<= 0.0.2"),
+        ]),
+        pkg!(("c0", "0.0.10") => [
+            dep_req("A", ">= 0.0.9, <= 0.0.10"),
+            dep_req("a-sys", "= 0.0.5"),
+        ]),
+        pkg!("j" => [
+            dep_req("A", ">= 0.0.3, <= 0.0.5"),
+            dep_req("R", ">=0.0.4, <= 0.0.6"),
+            dep_req("c", ">= 0.0.9"),
+            dep_req("c0", ">= 0.0.6"),
+        ]),
+    ];
+
+    let reg = registry(input.clone());
+    let _ = resolve_and_validated(&pkg_id("root"), vec![dep("j")], &reg);
+}
